@@ -5,17 +5,30 @@ import websocket
 
 from .event import EventType
 from .message import Message, PrivateMessage, MuteMessage
-from .errors import Banned, Throttled
+from .user import User
+from .errors import (
+    AccountTooYoung,
+    Banned,
+    DuplicateMessage,
+    InvalidMessage,
+    NeedLogin,
+    NoPermission,
+    NotFound,
+    ProtocolError,
+    SubMode,
+    Throttled,
+    TooManyConnections,
+)
 
 
 class DGGChat:
     WSS = "wss://chat.destiny.gg/ws"
     URL = "https://www.destiny.gg"
 
-    def __init__(self, auth_token=None, username: str = None):
+    def __init__(self, auth_token=None, username: str = None, wss: str = None):
         self.username = username.lower() if isinstance(username, str) else None
         self.ws = websocket.WebSocketApp(
-            self.WSS,
+            wss or self.WSS,
             cookie=f"authtoken={auth_token}" if auth_token else None,
             on_open=self._on_open,
             on_message=self._on_message,
@@ -24,6 +37,7 @@ class DGGChat:
         )
         self._connected = False
         self._events = {}
+        self._users = {}
 
     def __repr__(self):
         return (
@@ -32,6 +46,10 @@ class DGGChat:
             else f"{self.__class__.__name__}()"
         )
 
+    @property
+    def users(self) -> dict:
+        return self._users.copy()
+
     def _on_message(self, ws, message: str):
         event_type, data = message.split(maxsplit=1)
         data = json.loads(data)
@@ -66,6 +84,7 @@ class DGGChat:
             self.on_broadcast(msg)
         elif event_type == EventType.NAMES:
             logging.debug(f"{event_type} {data}")
+            self.on_names(data["connectioncount"], data["users"])
         elif event_type in (EventType.JOIN, EventType.QUIT):
             msg = Message(
                 self, event_type, data["nick"], data["features"], data["timestamp"]
@@ -105,6 +124,16 @@ class DGGChat:
                 data["duration"],
             )
             self.on_mute(msg)
+        elif event_type == EventType.UNMUTE:
+            msg = Message(
+                self,
+                event_type,
+                data["nick"],
+                data["features"],
+                data["timestamp"],
+                data["data"],
+            )
+            self.on_unmute(msg)
         elif event_type == EventType.REFRESH:
             msg = Message(
                 self, event_type, data["nick"], data["features"], data["timestamp"]
@@ -113,13 +142,24 @@ class DGGChat:
         elif event_type == EventType.PRIVMSGSENT:
             pass
         elif event_type == EventType.ERROR:
-            if data["description"] == "throttled":
-                raise Throttled
-            elif data["description"] == "banned":
-                raise Banned
+            err_dict = {
+                "banned": Banned,
+                "duplicate": DuplicateMessage,
+                "invalidmsg": InvalidMessage,
+                "needlogin": NeedLogin,
+                "nopermission": NoPermission,
+                "notfound": NotFound,
+                "privmsgaccounttooyoung": AccountTooYoung,
+                "protocolerror": ProtocolError,
+                "submode": SubMode,
+                "throttled": Throttled,
+                "toomanyconnections": TooManyConnections,
+            }
+            if (desc := data["description"]) in err_dict:
+                raise err_dict[desc]
             else:
                 logging.error(event_type, data)
-                raise Exception(data["description"])
+                raise Exception(desc)
         else:
             logging.warning(f"Unknown event type: {event_type} {data}")
 
@@ -136,7 +176,7 @@ class DGGChat:
         self._connected = False
 
     def _on_error(self, ws, error):
-        logging.error(error)
+        logging.error(error, exc_info=True)
 
     def event(self, event_name: str = None):
         """Decorator to run function when the specified event occurs."""
@@ -169,13 +209,21 @@ class DGGChat:
         for func in self._events.get("on_msg", tuple()):
             func(msg)
 
+    def on_names(self, connectioncount: int, users: list):
+        """Do stuff when the NAMES message is received upon connecting to chat."""
+        self._users = {
+            user["nick"].lower(): User(user["nick"], user["features"]) for user in users
+        }
+        for func in self._events.get("on_names", tuple()):
+            func(connectioncount, users)
+
     def on_privmsg(self, msg: PrivateMessage):
         """Do stuff when a PRIVMSG is received."""
         for func in self._events.get("on_privmsg", tuple()):
             func(msg)
 
-    def run(self):
-        self.ws.run_forever(origin=self.URL)
+    def run(self, origin: str = None):
+        self.ws.run_forever(origin=origin or self.URL)
 
     def send(self, msg: str):
         """Send a message to chat."""
@@ -194,11 +242,13 @@ class DGGChat:
 
     def on_join(self, msg):
         """Do stuff when chatter joins."""
+        self._users[msg.nick.lower()] = User(msg.nick, msg.features)
         for func in self._events.get("on_join", tuple()):
             func(msg)
 
     def on_quit(self, msg):
         """Do stuff when chatter joins."""
+        del self._users[msg.nick.lower()]
         for func in self._events.get("on_quit", tuple()):
             func(msg)
 
@@ -217,6 +267,11 @@ class DGGChat:
         for func in self._events.get("on_mute", tuple()):
             func(msg)
 
+    def on_unmute(self, msg: Message):
+        """Do stuff when a chatter is unmuted."""
+        for func in self._events.get("on_unmute", tuple()):
+            func(msg)
+
     def on_refresh(self, msg: Message):
         """Do stuff when refreshed."""
         for func in self._events.get("on_refresh", tuple()):
